const char* tutorial_en_jicfs2=\
"CategoryCode2,Category2\n\
11,Processed_Food\n\
12,Fresh_Food\n\
13,Snacks\n\
14,Alcoholic_Drinks\n\
19,Misc_Food\n\
21,Daily_Goods\n\
22,Medical\n\
23,Cosmetics\n\
24,Household_Appliances\n\
25,DIY_Supplies\n\
26,Pet_Supplies\n\
29,Misc_Supplies\n\
31,Office_Supplies\n\
32,Toys\n\
33,Books\n\
34,Musical_Instrument\n\
35,Sporting_Goods\n\
36,Info_Equipment\n\
39,Misc_Cultural_Supplies\n\
42,Furniture\n\
43,Car_Accessories\n\
44,Clock\n\
45,Optical_Products\n\
46,Home_Appliance\n\
49,Durable_Consumer_Goods\n\
51,Clothing\n\
53,Bedding\n\
54,Apparel\n\
55,Shoes\n\
56,Sporting_Goods\n\
59,Misc_Apparel\n\
99,Miscellanous\n";
