/* ////////// LICENSE INFO ////////////////////

 * Copyright (C) 2013 by NYSOL CORPORATION
 *
 * Unless you have received this program directly from NYSOL pursuant
 * to the terms of a commercial license agreement with NYSOL, then
 * this program is licensed to you under the terms of the GNU Affero General
 * Public License (AGPL) as published by the Free Software Foundation,
 * either version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTY, INCLUDING THOSE OF 
 * NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Please refer to the AGPL (http://www.gnu.org/licenses/agpl-3.0.txt)
 * for more details.

 ////////// LICENSE INFO ////////////////////*/
// =============================================================================
/// kg2crossHelp.h : kg2cross help
// =============================================================================
_titleL="1対Nのクロス集計";
_docL="\
\n\
M2CROSS 1対Nのクロス集計\n\
\n\
1対Nのクロス集計を行う。\n\
s=を指定した場合には項目の値が項目名となるように横に展開され、\n\
f=で指定した項目がセルとして出力される。 a=を指定した場合(2項目指定)には\n\
指定した値が項目名となり、 １項目にf=で指定した項目名が、\n\
２項目にf=で指定した項目値がそれぞれ縦展開される\n\
k=が指定されていた場合には、\n\
指定した値が行idとなり、id単位で展開される。\n\
";

