const char* tutorial_jp_jicfs2=\
"中分類,中分類名\n\
11,加工食品\n\
12,生鮮食品\n\
13,菓子類\n\
14,飲料・酒類\n\
19,その他食品\n\
21,日用雑貨\n\
22,医薬品\n\
23,化粧品\n\
24,家庭用品\n\
25,ＤＩＹ用品\n\
26,ペット用品\n\
29,その他日用品\n\
31,文具・事務用品・情報文具\n\
32,玩具\n\
33,書籍\n\
34,楽器・音響ソフト\n\
35,スポーツ用品\n\
36,情報機器\n\
39,その他文化用品\n\
42,家具\n\
43,車両用品\n\
44,時計・メガネ\n\
45,光学・写真関連品\n\
46,家電\n\
49,その他耐久消費財\n\
51,衣料・衣服\n\
53,寝具・寝装品\n\
54,身の回り品\n\
55,靴・履物\n\
56,スポーツ用品\n\
59,その他衣料・身の回り品\n\
99,その他商品\n";
