const char* tutorial_en_jicfs6=\
"CategoryCode6,Category6\n\
110101,Soy_Sauce\n\
110103,Sugar\n\
110105,Low_Calorie_Sweetener\n\
110107,Miso\n\
110109,Table_Salt\n\
110111,Vinegar\n\
110113,Vinegar_with_Sake(Japanese_Style)\n\
110115,Mirin_Seasoning\n\
110117,Cooking_Japanese_Wine\n\
110119,Cooking_Wine\n\
110121,Flavor_Seasoning\n\
110123,Broth\n\
110125,Single_Compound_Seasoning\n\
110129,Sauce\n\
110131,Ketchup\n\
110133,Barbecue_Meat_Sauce\n\
110135,Other_Sauce\n\
110137,Mayonnaises\n\
110139,Dressing\n\
110141,Seasonings(Other_Than_Mustard_Horseradish)\n\
110142,Mustard_Horseradish\n\
110143,Bonoito\n\
110149,Chinese_Seasoning\n\
110197,Other_Seasoning\n\
110203,Sesame_Oil\n\
110205,Tempura_Oil\n\
110207,Animal_Oil\n\
110209,Olive_Oil\n\
110211,Fragrant_Oil\n\
110297,Other_Cooking_Oil\n\
110301,Peanuts_Chocolate_Spread\n\
110303,Jam_Marmalade\n\
110397,Other_Spreads\n\
110401,Butter\n\
110403,Margarine_Fat\n\
110405,Cheese\n\
110407,Instant_Milk_Powder\n\
110409,Skim_Milk\n\
110411,Condensed_Milk\n\
110413,Fresh_Cream\n\
110415,Whipped_Cream\n\
110497,Other_Diary_Products\n\
110501,Instant_Curry\n\
110502,Raw_Curry\n\
110503,Instant_Stew\n\
110504,Raw_Stew\n\
110505,Sauce_Mixes\n\
110507,Chinese_Seasoning_Powder\n\
110509,Rice_Seasoning_Powder_Mix\n\
110511,Boiled_Rice_Additives\n\
110513,Range_Use\n\
110515,Sprinking\n\
110517,Salted_Vegetables_Seasoning\n\
110597,Other_Seasoning\n\
110601,Cooking_Soup\n\
110603,Instant_Soup\n\
110605,Instant_Miso_Soup\n\
110697,Other_Soup\n\
110701,Frozen_Seafood\n\
110703,Frozen_Meat\n\
110705,Frozen_Vegetables\n\
110707,Frozen_Food\n\
110721,Pizza_Gratins\n\
110723,Frozen_Noodles\n\
110725,Frozen_Rice_Topping\n\
110791,Ice\n\
110797,Other_Frozen_Food\n\
110801,Canned_Seafood(Except_Tuna)\n\
110802,Canned_Tuna\n\
110803,Canned_Vegetable\n\
110805,Canned_Fruits_Dessert\n\
110807,Canned_Meat\n\
110897,Other_Canned_Food\n\
111001,Wheat_Flour\n\
111003,Tempur_Flour\n\
111005,Bread_Crumbs\n\
111007,Chinese_Frying_Flour\n\
111009,Chestnut_Flour\n\
111011,Kinako\n\
111015,Rice_Flour\n\
111097,Other_Flour\n\
111101,Pre_Mix\n\
111105,Honey\n\
111107,Malt_Syrup\n\
111109,Syrup\n\
111111,Fruit_Peel_Extract\n\
111113,Dessert_Ingredients\n\
111197,Other_Home_Making_Ingredients\n\
111201,Instant_Sack_Noodle\n\
111203,Cup_Noodle\n\
111205,Dry_Noodle\n\
111207,Raw_Boiled_Noodle\n\
111209,Spaghetti\n\
111211,Macaroni\n\
111297,Other_Noodle\n\
111301,Bread\n\
111303,Sweet_Bread\n\
111305,Bread_For_Cooking\n\
111307,Cereals\n\
111397,Other_Bread\n\
111401,Rice\n\
111405,Package_Rice_Cake\n\
111497,Other_Grains\n\
111501,Meat_Ham\n\
111503,Meat_Sausage\n\
111505,Fish_Ham\n\
111507,Fish_Sausage\n\
111509,Barbecue_Pork\n\
111511,Beacon\n\
111597,Other_Processed_Meat\n\
111601,Boiled_Fish_Paste\n\
111603,Fish_Cake_Roll\n\
111605,Hanpan\n\
111607,Fried_Foods\n\
111697,Other_Processed_Food\n\
111701,Pickle\n\
111703,Boiled_Soy_Food\n\
111705,Rice_Bran_Ingredients_For_Pickle\n\
111797,Other_Pickle_Soy_Food\n\
111801,Tofu\n\
111803,Arum_Root\n\
111805,Fried_Tofu\n\
111807,Natto\n\
111897,Other_Water_Based_Food\n\
111901,Salad\n\
111903,Boiled_Beans\n\
111905,Japanese_Delicatessen\n\
111907,Chinese_Delicatessen\n\
111909,Western_Delicatessen\n\
111997,Other_Delicatessen\n\
112001,Sesame\n\
112003,Dried_Shiitake\n\
112005,Beans\n\
112097,Other_Dried_Agricultrual_Produce\n\
112101,Nori\n\
112105,Wakame\n\
112107,Kelp\n\
112109,Other_Seaweed\n\
112111,Bonito\n\
112113,Dried_Fish\n\
112197,Other_Processed_Seafood\n\
119701,Processed_Seafood\n\
119703,Processed_Seafood\n\
119705,Processed_Seafood\n\
119797,Other_Processed_Food\n\
120197,Other_Seafood\n\
120297,Other_Meat\n\
120397,Other_Produce\n\
129797,Other_Fresh_Food\n\
130121,Candies_Caramel\n\
130123,Chocolate\n\
130125,Toy\n\
130127,Biscuit_Cookie\n\
130129,Rice_Snack\n\
130131,Snack\n\
130133,Bean_Snack\n\
130135,Toy_Candy\n\
130137,Kogarasi\n\
130139,Semi_Kogarasi\n\
130141,Grilled_Candy\n\
130143,Candy_Set\n\
130198,Other_Candy\n\
130203,Desserts\n\
130205,Yogurt\n\
130297,Other_Desserts\n\
130301,Agricultural_Chinmi\n\
130303,Seafood_Chinmi\n\
130305,Meat_Chinmi\n\
130397,Other_Chinmi\n\
130401,Premium_Ice\n\
130403,Family_Ice\n\
130497,Other_Ice\n\
139797,Other_Candy\n\
140101,Instant_Coffee\n\
140103,Regular_Coffee\n\
140105,Cocoa\n\
140107,Black_Tea\n\
140109,Japanese_Tea\n\
140111,Barely_Tea\n\
140113,Chinese_Tea\n\
140115,Other_Tea\n\
140197,Other_Drinks\n\
140201,Pure_Juice\n\
140203,Juice\n\
140205,Juice_With_Pulp\n\
140209,Vegetables_Juice\n\
140211,Tomato_Juice\n\
140297,Other_Fruit_Drinks\n\
140301,Cola\n\
140305,Carbonic_Flavor\n\
140307,Coffee_Drink\n\
140309,Cocoa_Drink\n\
140311,Black_Tea_Drink\n\
140313,Japanese_Barley_Tea\n\
140315,Chinese_Tea\n\
140316,Other_Tea\n\
140317,Water\n\
140321,Carbonated_Water\n\
140323,Sport_Drink\n\
140325,Nutrition_Drink\n\
140327,Vinegar_Drink\n\
140397,Other_Cooling_Drink\n\
140401,Milk\n\
140403,Soy_Milk\n\
140405,Yogurt_Beverage\n\
140407,Yogurt_Bacillus_Beverage\n\
140497,Other_Yogurt_Drink\n\
140601,Sake\n\
140611,Synthetic_Sake\n\
140621,Rice_Wine_TypeA\n\
140623,Rice_Wine_TypeB\n\
140631,Mirin\n\
140641,Beer\n\
140651,Fruit_Wine\n\
140653,Sweet_Fruit_Wine\n\
140661,Whisky\n\
140663,Brandy\n\
140671,Spirits\n\
140681,Liquors\n\
140691,Sparkling_Wine\n\
140693,Powder_Liquor\n\
140695,Other_Cocktails\n\
140697,Other_Alcoholic_Drinks\n\
140701,Alcoholic_Beverage\n\
140797,General_Alcoholic_Beverage\n\
149797,Other_Drinks_Wine\n\
190101,Baby_Milk\n\
190103,Baby_Food\n\
190197,Other_Child_Milk_Products\n\
190201,Health_Food\n\
190203,Pregnant_Women_Food\n\
190297,Other_Health_Product\n\
190301,Food_Gift\n\
190397,Other_Food_Gift\n\
199701,Tobacco\n\
199797,Other_Food\n\
210101,Tooth_Polishing\n\
210103,Toothbrush\n\
210105,Mouthwash\n\
210107,Denture_Cleaners\n\
210197,Other_Oral_Care_Product\n\
210201,Toilet_Soap\n\
210203,Bath_Salts\n\
210205,Body_Shampoo\n\
210297,Other_Soap\n\
210301,Facial_tissue\n\
210303,Paper_Towel\n\
210305,Wet_Tissue\n\
210307,Toilet_Paper\n\
210309,Adult_Paper\n\
210311,Floral_Print_Tissue_Paper\n\
210313,Sanitary_Product\n\
210397,Other_Hygenic_Paper_Supplies\n\
210401,Qtips\n\
210403,Paper_Underwear\n\
210405,Contraceptives\n\
210407,Bandages_Gauze\n\
210409,Supporters\n\
210411,Masks_Eye_Cover\n\
210413,Abosrbent_Cotton\n\
210415,Adhesive_Tape\n\
210417,Scissors_Pincette\n\
210419,Thermometer\n\
210421,Hemadynamometer\n\
210423,Health_Meter\n\
210425,Contact_Lens_Solution\n\
210427,Medical_Magnetic_Products\n\
210497,Other_Hygienic_Medical_Supplies\n\
210501,Child_Use_Paper\n\
210503,Skin_Care\n\
210505,Nursing_Supplies\n\
210597,Other_Childcare_Supplies\n\
210601,Heavy_Duty_Laundry_Detergent\n\
210603,Light_Laundry_Detergent\n\
210605,Bleach\n\
210607,Softener\n\
210609,Finish_Medicine\n\
210611,Dyes\n\
210613,Laundry_Soap\n\
210697,Other_Farbic_Chemicals\n\
210701,Home_Detergent\n\
210703,Toilet_Detergent\n\
210705,Bus_Detergent\n\
210707,Glass_Detergent\n\
210709,Carpet_Cleaner\n\
210711,Pipe_Cleaner\n\
210713,Mold_Cleaner\n\
210715,Furniture_Cleaner\n\
210717,Scuttle_Fan_Range_Cleaner\n\
210719,Disposable_Paper_Cleaner\n\
210797,Other_Homeuse_Cleaners\n\
210801,Kitchen_Cleaner\n\
210803,Cleanser\n\
210805,Kitchen_Bleach\n\
210807,Oil_Processing_Detergent\n\
210897,Other_Kitchen_Cleaner\n\
210901,Aromatic_Cleaner\n\
210903,Air_Freshner\n\
210905,Aromatic_Toilet_Cleaner\n\
210907,Deodorant\n\
210909,Moisture_Removal\n\
210997,Other_Living_Envirnoment_Chemicals\n\
211001,Floor_Wax\n\
211003,Shoe_Cream\n\
211097,Other_Household_Products\n\
211101,Pestcide\n\
211103,Insecticide\n\
211105,Mosquito_Incense_Stick\n\
211107,Mosquito_Mat\n\
211109,Electronic_Mosquito_Remover\n\
211111,Cockroach_Trap\n\
211197,Other_Pestcides\n\
219701,Water_Repellant\n\
219703,Stain_Remover_Benzine\n\
219705,Bath_Water_Cleaner\n\
219707,Static_Remover\n\
219711,Eyewear_Cleaner\n\
219713,Disposal_Kairo\n\
219797,Other_Daily_Necessities\n\
220101,Ampuled_Liquid_Medicine\n\
220103,Mini_Ampuled_Medicine\n\
220105,Nutritious_Cardiacs\n\
220107,Strong_Liver_Medicine\n\
220109,Calcium\n\
220111,Women_Vitamins\n\
220113,Alcoholic_Medicine\n\
220115,Vitamin_B1\n\
220117,Vitamin_C\n\
220119,Vitamin_E\n\
220121,Multi_Vitamin\n\
220123,Other_Vitamin\n\
220197,Other_Nutrition_Supplement\n\
220201,Cold_Medicine\n\
220203,Nasitis_Medicine\n\
220205,Cough_Medicine\n\
220207,Troche_Medicine\n\
220209,Gargle\n\
220211,Pain_Reliever\n\
220213,Dizzy_Medicine\n\
220215,Hypnotic_Tranquilizer\n\
220217,Sleepiness_Inhibitor\n\
220219,Allergy\n\
220297,Other_Asthma_Neurotic_Medicine\n\
220301,Digestion\n\
220303,Diarrhea\n\
220305,Antacid\n\
220307,Enemas\n\
220309,Periodontoclasia_Remedy_Medicine\n\
220311,Hemorrhoids_Medicine\n\
220313,Anthelminthic\n\
220397,Other_Digestion_Medicine\n\
220401,Eye_Drops\n\
220403,Skin_Remedy\n\
220405,Skin_Sterilization\n\
220407,Athletes_Foot_Medicine\n\
220409,Poultice_Medicine\n\
220411,Anesthetic_External_Use\n\
220413,Hair_Agent\n\
220415,Skin_Disease_Internal_Use\n\
220497,Other_Skin_Senses_Medicine\n\
220501,Chinese_Herb_Medicine\n\
229701,Cardiac\n\
229703,Other_Circulatory_Medicine\n\
229705,Child_Medicine\n\
229707,Contraceptive\n\
229709,Diagnostic_Medicine\n\
229711,Insecticide_Medical\n\
229713,Prep_Medicine\n\
229715,Professional_Medicine\n\
229797,Other_Medical_Supplies\n\
230101,Make_Up_Remover\n\
230103,Skin_Lotion\n\
230105,Cold_Cream\n\
230107,Hand_Cream\n\
230109,Milky_Lotion\n\
230111,Make_Up_Lotion\n\
230113,Pack\n\
230115,Body_Use\n\
230117,Cleaning_Solution\n\
230121,Lip_Balm\n\
230123,Beauty_Care_Liquid\n\
230197,Other_Basic_Cosmetics\n\
230201,Oshiroi\n\
230203,Eye_Liner\n\
230205,Eye_Shadow\n\
230207,Mascara\n\
230209,Foundation\n\
230211,Make_Up_Foundation\n\
230213,Lipstick\n\
230215,Manicure\n\
230217,Powders\n\
230219,Eye_Shadow\n\
230297,Other_Make_Up\n\
230301,Perfume\n\
230303,Colons\n\
230305,Antiperspirants\n\
230397,Other_Fragrance\n\
230401,Hair_Cream\n\
230403,Hair_Tonic\n\
230405,Hair_Spray\n\
230407,Hair_Styling\n\
230409,Hair_Color\n\
230411,Shampoo\n\
230413,Hair_Rinse\n\
230415,Hair_Treatment\n\
230417,Brushing_Agent\n\
230419,Blowing_Agent\n\
230421,Perm_Agent\n\
230423,Hair_Regrowth\n\
230497,Other_Hair_Care\n\
230501,Shaving_Cream_Foam\n\
230503,Skin_Lotion_Milk\n\
230505,After_Shave_Lotion\n\
230507,Hair_Liquid\n\
230509,Styling_Agent\n\
230511,Pomade_Check\n\
230513,Check\n\
230515,Hair_Restorer\n\
230517,Hair_Tonic\n\
230519,Hair_Cream\n\
230521,Hair_Spray\n\
230523,Fragrance\n\
230525,Make_Up\n\
230597,Other_Men_Cosmetics\n\
230601,Razors\n\
230603,Hair_Make_Up_Accessories\n\
230605,Face_Make_Up_Accessories\n\
230607,Body_Make_Up_Accessories\n\
230697,Other_Make_Up_Accessories\n\
239797,Other_Make_Up\n\
240101,Lapping_Film\n\
240103,Aluminum_Foils\n\
240197,Other_Food_Packaging\n\
240201,Garbage_Bag\n\
240203,Broom\n\
240205,Dusting\n\
240207,Cleaning_Brush\n\
240209,Mops_Cloth\n\
240211,Bucket\n\
240213,Waste_Bin\n\
240297,Other_Cleaning_Supplies\n\
240301,Laundry_Supplies\n\
240303,Drying_Supplies\n\
240305,Laundry_Finish_Supplies\n\
240397,Other_Laundary_Finish_Supplies\n\
240401,Gas_Mat\n\
240403,Pot_Dish_Scrubber\n\
240405,Scrubbing_Brush_Sponge\n\
240407,Paper_Filter\n\
240409,Can_Opener_Corkscrews\n\
240411,Sink_Supplies\n\
240413,Kitchen_Papers\n\
240497,Other_Kitchen_Supplies\n\
240501,Bathroom_Supplies\n\
240503,Face_Wash_Supplies\n\
240505,Body_Wash_Supplies\n\
240507,Toilet_Supplies\n\
240597,Other_Bathroom_Toilet_Supplies\n\
240601,Seasoning_Container\n\
240603,Tabletop_Accessories\n\
240605,Chopstick\n\
240607,Knives_Forks_Spoon\n\
240609,Bowl\n\
240611,Plate\n\
240613,Bowl\n\
240615,Glass\n\
240617,Teacup_Teapot\n\
240619,Clay_Bottle_Iron_Kettle\n\
240621,Liquor_Bottle\n\
240623,Simple_Tableware\n\
240625,Lunchbox\n\
240627,Thermos\n\
240697,Other_Tabletop_Accessories\n\
240701,Pot_Kettles\n\
240703,Kettles\n\
240705,Frying_Pans\n\
240707,Cooking_Container\n\
240709,Confectionery_Supplies\n\
240797,Other_Cooking_Utilities\n\
240801,Child_Seat\n\
240803,Wallpaper\n\
240805,Hanger_Hook\n\
240807,Hanger\n\
240809,Living_Supplies\n\
240811,Foldable_Case\n\
240813,Tablecloths\n\
240897,Other_Living_Supplies\n\
249701,Insense_Stick\n\
249703,Candle\n\
249705,Glove\n\
249707,Match\n\
249709,Lighter\n\
249711,Smoking_Accessories\n\
249713,Packing_Supplies\n\
249715,Home_Use_Bags\n\
249717,Handicraft_Needlework_Supplies\n\
249719,Shoe_Cleaning_Supplies\n\
249797,Other_Household_Supplies\n\
250101,Carpenter_Supplies\n\
250103,Power_Tools\n\
250105,Paint_Equipments\n\
250197,Other_Construction_Painting_Tools\n\
250201,Construction_Metallic_Parts\n\
250203,Lumber\n\
250205,Basic_Materials\n\
250207,Painting_Materials\n\
250209,Adhesive\n\
250211,Repair_Supplies\n\
250297,Other_Construction_Painting_Material\n\
250301,Gas_Part\n\
250303,Aqueduct_Part\n\
250397,Other_Piping_Supplies\n\
250401,Insecticide\n\
250403,Fertilizer\n\
250405,Concrete\n\
250407,Bulbs_Seeds\n\
250409,Flower_Bowls_Stands\n\
250411,Garden_Tools\n\
250413,Flower_Supplies\n\
250497,Other_Gardening_Supplies\n\
259797,Other_DIY_Supplies\n\
260101,Dog_Food\n\
260103,Cat_Food\n\
260105,Bird_Food\n\
260107,Fish_Food\n\
260197,Other_Pet_Food\n\
260201,Pet_Medicine\n\
260203,Pet_Shampoo\n\
260297,Other_Pet_Cleaning_Agent\n\
260301,Dog_Cat_Toys\n\
260303,Bird_Toys\n\
260305,Fish_Toys\n\
260397,Other_Pet_Toys\n\
269797,Other_Pet_Supplies\n\
290101,Daily_Necessities_Gift\n\
290197,Other_Daily_Necessities\n\
290301,Photography_Film\n\
290303,8mm_Film\n\
290397,Other_Film\n\
290401,Cassette_Tape\n\
290403,Open_Reel_Tape\n\
290405,Video_Tape\n\
290497,Other_Audio_Video_Tape\n\
299797,Other_Daily_Necessities\n\
310101,OA_Stand\n\
310103,Cardcase_Box\n\
310197,Information_Organizers\n\
310201,File\n\
310203,Box_File\n\
310205,Folders\n\
310207,Clipboard\n\
310209,Binder\n\
310297,General_File_Binder\n\
310301,Seals_Printing_Supplies\n\
310303,Fastener\n\
310305,Clip\n\
310307,Push_Pin\n\
310309,Rubber_Band\n\
310311,Cutter_Knife\n\
310313,Scissors\n\
310315,Adhesive_Tape\n\
310317,Self_Adhesive_Memo\n\
310319,Glue\n\
310397,General_Office_Supplies\n\
310401,Pencil\n\
310403,Mechanical_Pencil\n\
310405,Ball_Pen\n\
310407,Marker\n\
310409,Fountain_Pen\n\
310411,Eraser\n\
310413,Correction_Fluid\n\
310415,Writing_Pen\n\
310497,General_Writing_Equipment\n\
310501,Notebook\n\
310503,Memo_Pad\n\
310505,Card\n\
310507,Stationary\n\
310509,Envelope\n\
310511,Filer\n\
310513,Office_Use_Paper_Product\n\
310515,Notification_Paper\n\
310597,General_Paper_Product\n\
310601,Album\n\
310603,Ceremony_Use_Supplies\n\
310605,General_Paper_Product\n\
310607,Import_Paper_Products\n\
310609,Postcard\n\
310611,Japanese_Book\n\
310613,Packaging_Supplies\n\
310697,General_Daily_Product\n\
310701,Picture_Supply\n\
310703,Caligraphy_Supply\n\
310705,Sculpture_Supply\n\
310707,Soroban\n\
310709,Pencil_Sharpener\n\
310711,Pencil_Case\n\
310713,Compass_Rulers\n\
310715,Teaching_Aid\n\
310717,Globe\n\
310719,Optical_Supplies\n\
310797,General_Education_Related_Supplies\n\
310801,Calculator\n\
310803,Electronic_Organizer\n\
310805,Electronic_Board\n\
310897,General_Electronic_Stationary\n\
310901,Checkwriter\n\
310903,Punches\n\
310905,Stapler\n\
310907,Scissors\n\
310997,General_Office_Equipment\n\
311001,Word_Processor\n\
311003,Copier\n\
311005,Clocks\n\
311007,Facsimile_Telephone\n\
311009,Register\n\
311011,Personal_Computer\n\
311013,Printer\n\
311015,Shredder\n\
311017,Laminater\n\
311019,Overhead_Projector\n\
311021,Binding_Machine\n\
311097,General_Business_Machine\n\
311101,Data_Files_Binder\n\
311103,Floppy_Disk\n\
311105,OA_Paper\n\
311107,OA_Ribbon\n\
311109,Laminating_Supply\n\
311111,OHP_Supply\n\
311197,General_OA_Price\n\
311201,Drafting_Machine\n\
311203,Drafting_Pen\n\
311205,Drafing_Compass_Ruler\n\
311297,General_Drafting_Design_Supply\n\
311301,Lettering_Set\n\
311397,General_Picture_Design_Material\n\
311501,Desk_Table\n\
311503,Chair\n\
311505,Foldable_Furniture_Locker\n\
311507,Safe\n\
311509,Goods_Shelf\n\
311511,Office_Accessories\n\
311513,Blackboard_Whiteboard\n\
311515,Reception_Set\n\
311597,General_Office_Furniture\n\
311601,Bag_Sack\n\
311603,Pass_Case\n\
311605,Card_Case\n\
311697,General_Portable_Stationary\n\
311797,Stationary_General_Office_Supply_Gift\n\
311801,Battery\n\
311803,Cassette\n\
311805,Videotape\n\
311897,General_Stationary_Electronics_Supply\n\
319797,Other_Stationary_Business_Information_Supplies\n\
320101,Board_Game\n\
320103,Action_Game\n\
320105,Mini_Game\n\
320107,Joke_Game\n\
320111,Simulation_Game\n\
320113,Card_Game\n\
320115,Classical_Game\n\
320117,Magic_Tricks\n\
320119,Magic_Toys\n\
320121,Other_Adult_Game\n\
320197,Miscellaneous_Game\n\
320201,Puzzle\n\
320203,Jigsaw_Puzzle\n\
320297,Other_Puzzle\n\
320301,Electronic_Game\n\
320397,Other_Electronic_Game\n\
320401,TV_Game_Personal_Computer\n\
320403,TV_Game_PC_Software\n\
320497,Other_TV_PC_Game\n\
320501,Original_Boy_Character\n\
320503,War_Boy_Character\n\
320597,Other_Boy_Character\n\
320601,Friction_Infant_Toy\n\
320603,Battery_Powered_Infant_Toy\n\
320605,Voice_Infant_Toy\n\
320697,Other_Infant_Toy\n\
320701,Mini_Car\n\
320703,Mini_Car_Accessories\n\
320705,Action_Mini_Car\n\
320797,Other_Mini_Car\n\
320801,Rail_Toy\n\
320897,Other_Rail_Toy\n\
320901,Radio_Control_Toy\n\
320903,Remote_Control\n\
320905,Racing\n\
320909,Robots\n\
320911,Science_Hobby\n\
320997,Other_Science_Toy\n\
321001,Doll\n\
321003,House_Furniture\n\
321005,Doll_Accessories\n\
321097,Other_Doll\n\
321101,Miniture_Untensils_Food\n\
321197,Other_Miniture_Untensils_Food\n\
321201,Girl_Hobby\n\
321203,Girl_Collection_Hobby\n\
321207,Music\n\
321297,Other_Girl_Hobby\n\
321301,Huggables\n\
321303,Action_Doll\n\
321305,Fashion_Doll\n\
321397,Other_Doll\n\
321401,Educational_Toys\n\
321403,Block\n\
321405,Building_Block\n\
321407,Microphones_Musical_Instrument\n\
321409,Construction\n\
321411,Child_Intellectual_Growth\n\
321497,Other_Child_Intellectual_Growth\n\
321501,Water_Floater\n\
321503,Puzzle_Toy\n\
321505,Puzzle_Blocks\n\
321597,Other_Nurture_Toy\n\
321601,Living\n\
321603,Daily_Miscellanous_Goods\n\
321605,Close_Miscellanous_Goods\n\
321607,Food\n\
321609,Party\n\
321611,Leisure_Supplies\n\
321613,Stationary\n\
321697,Other_Fancy\n\
321701,Sewing\n\
321703,Character_Sewing\n\
321705,Action_Sewing\n\
321797,Other_Sewing\n\
321801,Plastic_Model_Land\n\
321803,Plastic_Model_Sea\n\
321805,Plastic_Model_Air\n\
321807,Plastic_Model_Character\n\
321897,Other_Plastic_Model\n\
321901,Radio_Control_Electromotive\n\
321903,Engine_Radio_Control\n\
321905,Radio_Control_Supplies\n\
321997,Other_Radio_Control\n\
322001,N_Gage\n\
322003,Other_Gauge\n\
322005,Railroad_Model_Supplies\n\
322097,Other_Railroad_Model\n\
322101,Shooting_Gun\n\
322103,Model_Gun\n\
322105,Gun_Supplies\n\
322197,Other_Gun\n\
322201,Craft_Hobby\n\
322203,Adult_Hobby\n\
322297,Other_Craft_Adult_Hobby\n\
322301,Hobby_Related_Supplies\n\
322397,Other_Hobby_Related_Supplies\n\
322401,Japanese_Character\n\
322403,Foreign_Character\n\
322405,Adult_Character\n\
322497,Other_Gift_Character\n\
322501,New_Year_Doll\n\
322503,March_Doll\n\
322505,May_Doll\n\
322507,Other_Doll\n\
322509,New_Year_Supplies\n\
322511,Water_Vinyl_Supplies\n\
322513,Other_Summer_Supplies\n\
322515,Festival_Day_Toy\n\
322517,Christmas_Item\n\
322597,Other_Seasonal_Item\n\
322601,Child_Fireworks\n\
322603,Youth_Fireworks\n\
322697,Other_Fireworks\n\
322701,Small_Toys\n\
322703,Paper_Toys\n\
322797,Other_Small_Toys\n\
322801,Toy_Sport\n\
322803,Junior_Toy_Sport\n\
322897,Other_Sport\n\
322901,Car\n\
322903,Baby_Car\n\
322905,Baby_Trolley\n\
322997,Other_Car\n\
329797,Other\n\
339797,General_Book\n\
340101,Piano\n\
340103,Harpsichord_String_Instrument\n\
340105,Harpsichord\n\
340197,Other_Piano_Key_Instrument\n\
340201,Pipe_Organ\n\
340203,Reed_Organ\n\
340205,Hamonica\n\
340207,Keyboard_Hamonica\n\
340209,Accordion\n\
340211,Panpipe\n\
340297,General_Lead_Musical_Instrument\n\
340301,Brass_Instrument\n\
340303,Wooden_Musical_Instrument\n\
340397,General_Woodwind_Instrument\n\
340401,Violin\n\
340403,Viola\n\
340405,Cello\n\
340407,Double_Bass\n\
340409,Mandolin\n\
340411,Acoustic_Guitar\n\
340413,Ukulele\n\
340415,Banjo\n\
340497,General_String_Instrument\n\
340501,Taiko\n\
340503,Timpani\n\
340505,Stand_Xylophone\n\
340507,Koto\n\
340509,Tubular_Bell\n\
340511,Gong\n\
340513,Cymbal\n\
340515,Tambourine\n\
340597,General_Percussion_Instrument\n\
340601,Electric_Instrument\n\
340603,Electronic_Instrument\n\
340697,General_Electronic_Instrument\n\
340701,String_Instrument\n\
340703,Wind_Instrument\n\
340705,Percussion_Instrument\n\
340797,General_Japanese_Instrument\n\
340801,String_for_Instrument\n\
340803,Bow_for_Instrument\n\
340805,Bowls\n\
340807,Automatic_Play_Musical_Instrument\n\
340809,Music_Box\n\
340811,Metronome\n\
340813,Tuning_Fork\n\
340897,General_Musical_Instrument_Accessory\n\
340901,Disk_Recording_Supply\n\
340903,Tape_Recording_Supply\n\
340905,Film_Recording\n\
340997,General_Recording\n\
349797,Other_Musical_Instrument\n\
360101,Central_Processing_Unit\n\
360103,Personal_Computer\n\
360105,Auxiliary_Memory\n\
360107,Input_Unit\n\
360109,Terminal_Unit\n\
360111,Offline_Device\n\
360113,Calculation_Input_Accessory\n\
360197,General_Digital\n\
360297,General_Analog\n\
360397,General_Hybrid\n\
360401,General_Purpose_Program\n\
360403,Custom_Program\n\
360497,General_Systems_Program\n\
360501,General_Purpose_Program\n\
360503,Custom_Program\n\
360597,General_Applications_Program\n\
360601,Telephone\n\
360603,Telephone_Machinery\n\
360605,Telegraphic_Device\n\
360607,Pictorial_Device\n\
360609,Switchboard\n\
360611,Transmission_Device\n\
360613,Wire_Communication_Device_Accessories\n\
360697,General_Wired_Communication_Device\n\
369797,Other_Communication_Device\n\
390101,Match\n\
390103,Match_Boxes\n\
390105,Attached_Wood\n\
390107,Candlies\n\
390109,Miscellanous_Things\n\
390111,Incense_Stick\n\
390197,General_Matches\n\
390201,Button\n\
390203,Press_Snap_Fastener\n\
390205,Needling_Accessories\n\
390297,Needlework_Tools\n\
390301,Traffic_Sign\n\
390303,Parking_Sign\n\
390305,Advertisement_Supplies\n\
390307,Electric_Signs\n\
390309,Ad_Balloon\n\
390311,Projector\n\
390397,General_Label_Advertisement_Supplies\n\
390401,Worship_Shelves_Accessories\n\
390403,Butsudan\n\
390405,Festive_Worship_Supplies\n\
390407,Burial_Supplies\n\
390497,General_Religious_Supplies\n\
399797,Other_Cultural_Supplies\n\
420101,Japanese_Cabinet\n\
420103,Western_Cabinet\n\
420105,Drawers_Chest\n\
420197,General_Cabinet\n\
420201,Decorative_Cabinet\n\
420203,Book_Closets\n\
420205,Sideboard\n\
420207,Cabinet\n\
420209,Tea_Shelf\n\
420211,Organizer_Closets\n\
420213,Display_Closets\n\
420215,Black_Boxes\n\
420297,General_Closet\n\
420301,Bookshelf\n\
420303,Decoration_Shelf\n\
420305,Display_Shelf\n\
420307,Organizer_Shelf\n\
420397,General_Shelf\n\
420401,Clothing_Organizer\n\
420403,Needlework_Box\n\
420405,Toy_Box\n\
420407,General_Box\n\
420409,Office_Use_Box\n\
420497,General_Box\n\
420501,Radio_Cabinet\n\
420503,Television_Cabinet\n\
420505,Stereo_Cabinet\n\
420597,General_Cabinet\n\
420601,Desk\n\
420603,Stand_Desk\n\
420605,School_Desk\n\
420697,General_Desk\n\
420701,Table\n\
420703,Stand_Table\n\
420797,General_Table\n\
420801,One_Sided_Dresser\n\
420803,Triple_Sided_Dresser\n\
420805,Princess_Dresser\n\
420807,Full_Body_Mirror\n\
420809,Wall_Mount_Mirror\n\
420811,Dresser\n\
420897,General_Dresser\n\
420901,Kitchen_Table\n\
420903,Tray_Stand\n\
420905,Sink\n\
420907,Gas_Stand\n\
420909,Benchboard\n\
420911,Laboratory_Table\n\
420913,Display_Stand\n\
420915,Counter_Stand\n\
420997,General_Stand\n\
421001,One_Person_Chair\n\
421003,Stool\n\
421005,Chair\n\
421007,Small_Chair\n\
421009,Arm_Chair\n\
421011,Long_Chair\n\
421013,Sleep_Chair\n\
421015,Baby_Chair\n\
421017,Special_Uses_Chair\n\
421097,General_Chair\n\
421101,Bed\n\
421103,Hollywood_Bed\n\
421105,Two_Fold_Bed\n\
421107,Baby_Bed\n\
421109,Standard_Bed\n\
421111,Convertible_Bed\n\
421113,Special_Uses_Bed\n\
421115,Bedding_Supplies\n\
421197,General_Bed\n\
421201,Spring_Mattress\n\
421203,Foam_Mattress\n\
421205,Water_Mattress\n\
421297,General_Mattress\n\
421301,Fire_Resistant_Safe\n\
421303,Handbag_Safe\n\
421397,General_Safe\n\
421401,Clothing_Locker\n\
421403,Item_Locker\n\
421497,General_Locker\n\
421501,Vase_Stand\n\
421503,Garden_Plant_Stand\n\
421505,Telephone_Stand\n\
421507,Ornament_Stand\n\
421509,Television_Stand\n\
421511,Shogi_Board\n\
421513,Viewing_Stand\n\
421597,General_Stand\n\
429797,Other_Furniture\n\
430101,Car_Care\n\
430103,Car_Interior_Accessories\n\
430105,Car_Exterior_Accessories\n\
430107,Car_Tools\n\
430197,General_Car_Supplies\n\
430301,Family_Car\n\
430303,Sports_Car\n\
430305,Child_Car\n\
430307,Car_Part\n\
430397,General_Car\n\
430501,Bicycle_Part\n\
430597,General_Bicycle\n\
439797,Other_Car_Accessories\n\
439901,Car_Alarm_Clock\n\
439903,Car_Deodorant\n\
439997,Other_Car_Accessories\n\
440101,Watch\n\
440103,Travel_Clock\n\
440105,Panel_Use_Clock\n\
440107,Positioning_Clock\n\
440109,Hanging_Clock\n\
440197,General_Clock\n\
440201,Watch_Case\n\
440203,Clock_Case_Part\n\
440205,Other_Clock_Part\n\
440297,General_Clock_Related_Supplies\n\
440501,Spectacle_Lens\n\
440510,Spectacle_Related_Item\n\
440597,General_Spectacle\n\
449797,Other_Clock_Spectacle\n\
450101,Compact_Camera\n\
450103,35mm_Reflex_Camera\n\
450105,Medium_Format_Camera\n\
450107,Instant_Camera\n\
450109,Movie_Video_Camera\n\
450111,Small_Body_Camera\n\
450197,Other_Camera\n\
450201,35mm_Interchangeable_Lens\n\
450203,Medium_Format_Interchangeable_Lens\n\
450297,Other_Interchangeable_Lens\n\
450301,Binoculars\n\
450303,Telescope\n\
450305,Microscope\n\
450397,Other_Optical_Equipment\n\
450401,Filter\n\
450403,Strobo\n\
450405,Tripod\n\
450407,Bag\n\
450409,Album\n\
450411,Battery\n\
450413,Sliding_OHP\n\
450415,Auxiliary_Lens\n\
450417,Illumination_Supplies\n\
450419,Photographing_Supplies\n\
450421,Darkroom_Supplies\n\
450497,Other_Photographic_Accessories\n\
450501,Negative_Color_Film\n\
450503,Lens_Attaching_Film\n\
450505,Reversal_Color_Film\n\
450507,Monochrome_Film\n\
450509,Instant_Film\n\
450511,Special_Film\n\
450597,Other_Film\n\
459797,Other_Optical_Photographic_Supplies\n\
460105,Color_Television\n\
460110,HDTV\n\
460115,Projection_Television\n\
460120,LCD_Television\n\
460197,Television_Related_Equipment\n\
460305,Video_Deck\n\
460310,Integrated_Camera_Video\n\
460397,Other_Television_Related_Equipment\n\
460505,Video_Disc\n\
460597,VPD_Related\n\
460705,Amplifier\n\
460710,Tuner\n\
460715,Analog_Player\n\
460720,Speaker\n\
460725,Tape_Deck\n\
460730,CD_Player\n\
460735,Digital_Device\n\
460797,Component_Systems_Other_Stereo\n\
460905,Mini_Stereo_Systems\n\
460997,Other_Stereo\n\
461105,Microphone\n\
461110,Headphone\n\
461115,Audio_Rack\n\
461120,Equalizer\n\
461125,Audio_Timer\n\
461197,Other_Audio\n\
461305,Radio\n\
461310,Radio_Cassette_Recorder\n\
461315,Stereo_Headphone\n\
461320,Tape_Recorder\n\
461397,Telco_Other_Radio\n\
461505,Car_Stereo\n\
461510,Car_Amplifier\n\
461515,Car_Speaker\n\
461520,Car_CD\n\
461597,Other_Car_Components\n\
461705,Transceiver\n\
461710,Personal_Radios\n\
461715,Amplifier\n\
461797,Transceiver_Other_Amplifier\n\
461905,Videotape\n\
461910,Audio_Tape\n\
461915,Floppy_Disk\n\
461920,MD_DCC\n\
461925,Floppy_Software\n\
461930,Optical_Disk\n\
461935,Computer_Media_Group\n\
461997,Magnetic_Tape\n\
462105,Room_Air_Con\n\
462110,Business_Use_Air_Con\n\
462197,Other_Air_Con\n\
462305,Electric_Fan\n\
462310,Cold_Air_Fan\n\
462397,Other_Electric_Fan\n\
462505,Kerosene_Space_Heater\n\
462510,Petroleum_Fan_Heater\n\
462515,Petroleum_Warm_Air_Heater\n\
462520,Gas_Heater\n\
462525,Gas_Fan_Heater\n\
462597,Other_Room_Heating\n\
462705,Electronic_Blanket\n\
462710,Furniture_Pitch\n\
462715,Yagracotats\n\
462720,Electronic_Carpet\n\
462725,Electric_Radiant_Heater\n\
462730,Anchors_Cairo\n\
462735,Foot_Warmer\n\
462740,Humidifier\n\
462745,Portable_Heater_Related\n\
462797,Other_Portable_Heater\n\
462905,Water_Heater\n\
462910,Gas_Table\n\
462915,Gas_Cooking_Device\n\
462920,Scuttle_Fan\n\
462925,Range_Hood_Fan\n\
462930,Dish_Dryer\n\
462935,Dish_Washer\n\
462997,Other_Kitchen_Appliance\n\
463105,Microwave_Oven\n\
463110,Build_In_Oven\n\
463115,Kitchen_Stove\n\
463120,Electromagnetic_Cooking_Device\n\
463197,Other_Ranges\n\
463305,Juicer_Mixer\n\
463310,Cake_Macine\n\
463315,Hand_Mixer\n\
463320,Coffee_Mill\n\
463325,Electric_Ice_Shaver\n\
463330,Sharpener\n\
463397,Other_Rotary_Cooking_Device\n\
463505,Electric_Rice_Cooker\n\
463510,Electronic_Jar\n\
463515,Toaster\n\
463520,Electric_Oven\n\
463525,Coffee_Maker\n\
463530,Hot_Plate\n\
463535,Timer_Switch_Group\n\
463597,Other_Electronic_Cooking_Device\n\
463705,Water_Purifiers\n\
463797,Other_Cooking_Related\n\
463905,Refrigerator\n\
463910,Freezer\n\
463915,Water_Cooler\n\
463997,Other_Refrigeration_Device\n\
464105,Washer\n\
464110,Dryer\n\
464115,Futon_Dryer\n\
464120,Stand\n\
464125,Vaccum_Cleaner\n\
464197,Other_Housekeeping_Supplies\n\
464305,Hair_Dryer\n\
464310,Hot_Curler\n\
464315,Electric_Razor\n\
464320,Iron\n\
464325,Pants_Presser\n\
464397,Other_Beauty_Care_Supplies\n\
464505,Massage_Chair\n\
464510,Vibrator\n\
464515,Hearing_Aid\n\
464520,Clinical_Thermometer\n\
464530,Hemadynamometers\n\
464535,Low_High_Frequency_Medical_Device\n\
464540,Electric_Toothbrush\n\
464545,Buccal_Cavity_Washer\n\
464550,Training_Instrument\n\
464555,Golf_Training_Machine\n\
464560,Foam_Home_Sauna\n\
464565,Toilet_Washer\n\
464597,Other_Health_Medical_Equipment\n\
464705,Pendant\n\
464710,Sealing_Light\n\
464715,Bracket\n\
464720,Chandelier\n\
464725,Stands_With_Floor_Lamp\n\
464797,Other_Illumination_Supplies\n\
464905,Calculator\n\
464910,Electric_Translator\n\
464915,Electric_Note\n\
464920,School_Supplies\n\
464997,Other_Business_Machines_School_Supplies\n\
465105,Clock\n\
465110,Camera\n\
465197,Other_Clock_Camera\n\
465305,Pianos_Home_Appliance\n\
465310,Electronic_Pianos_Home_Appliance\n\
465315,Electronic_Organs_Home_Appliance\n\
465320,Synthesizers_Home_Appliance\n\
465397,Other_Instruments\n\
465505,Personal_Computer\n\
465510,Telephones\n\
465515,Professional_Writer\n\
465520,Copier\n\
465525,Facsimile\n\
465597,Other_OA_Equipment\n\
465705,Kitchen_Equipment\n\
465710,Pump\n\
465715,Face_Washing_Supplies\n\
465720,Calorifiers\n\
465725,Interphones_Chime\n\
465797,Other_Residence_Facilities\n\
465905,Electromotive_Tool\n\
465910,Gardening_Tool\n\
465915,Portable_Power_Tools\n\
465997,Other_Gardening_Carpenters_Leisure_Supplies\n\
466105,Battery\n\
466110,Light_Bulb\n\
466115,Fluorescent_Lamp\n\
466120,Wiring_Accessories\n\
466125,Flashlights\n\
466130,Electric_Battery_Cleaner\n\
466197,Other_Home_Appliance\n\
466305,Subsititution_Blade\n\
466310,Vaccum_Cleaner_Bag\n\
466315,Kerosene_Heater_Core\n\
466320,Scuttle_Fan_Filter\n\
466325,Air_Cleaner_Filter\n\
466330,Printer_Ribbon\n\
466335,Word_Processor_Paper\n\
466340,Thermal_Paper\n\
466345,Label\n\
466350,Water_Purifier_Filter\n\
466397,Other_Consumables\n\
469797,Other_Appliances\n\
490101,Picture\n\
490103,Sculpture\n\
490105,Bookmarks\n\
490197,General_Fine_Arts\n\
490301,Sewing\n\
490397,General_Sewing\n\
490497,General_Precious_Metal\n\
499797,Other_Durable_Consumer_Goods\n\
510101,Child_Underwear\n\
510103,Gentleman_Underwear\n\
510105,Lady_Underwear\n\
510107,Lingeries_Foundation\n\
510109,Nightwear\n\
510111,Child_Sock\n\
510113,Gentleman_Sock\n\
510115,Lady_Sock\n\
510197,General_Underwear_Sock\n\
510201,Baby_Supplies\n\
510203,Young_Child_Clothes\n\
510205,Young_Child_Goods\n\
510207,Boy_Clothes\n\
510209,Boy_Goods\n\
510211,Girl_Clothes\n\
510213,Girl_Goods\n\
510215,School_Clothing\n\
510297,General_Child_Clothing\n\
510301,Mens_Wear\n\
510303,Coats\n\
510305,Tops\n\
510307,Sweaters_Base\n\
510309,Slacks\n\
510311,Shirt\n\
510313,Working_Wear\n\
510397,General_Gentleman_Clothing\n\
510401,Lady_Clothing\n\
510403,Coat\n\
510405,Skirt\n\
510407,Sweaters\n\
510409,Slacks\n\
510411,Material\n\
510413,Fur\n\
510415,Supplies\n\
510497,General_Lady_Clothing\n\
510501,Ski\n\
510503,Tennis\n\
510505,Golf\n\
510507,Marine\n\
510509,Fitness\n\
510511,Mountain_Climbing\n\
510597,General_Sportswear\n\
510601,Feather_Down\n\
510603,Dyed_Shaku\n\
510605,Woven_Shaku\n\
510607,Band\n\
510609,Practical_Draperies\n\
510611,Small_Articles_Accessories\n\
510613,Mourning_Equipment\n\
510615,Coat\n\
510697,General_Drapery\n\
510701,Gentleman_Tux\n\
510703,Gentleman_Formal_Wear\n\
510705,Gentleman_Top_Coats_Slacks\n\
510707,Gentleman_Shirt\n\
510709,Lady\n\
510711,Boy\n\
510713,Girl\n\
510715,Small_Accessories\n\
510797,General_Formal_Wear\n\
510801,Gentleman_Denim_Jeans\n\
510803,Gentleman_Color_Jeans\n\
510805,Gentleman_Other_Bottoms\n\
510807,Gentleman_Jeans_Top\n\
510809,Woman_Denim_Jeans\n\
510811,Woman_Color_Jeans\n\
510813,Woman_Other_Bottoms\n\
510815,Woman_Jeans_Top\n\
510817,Child_Jeans\n\
510819,Child_Jenas_Top\n\
510897,General_Jeans\n\
519797,Other_Clothing\n\
530101,Futon_Cover\n\
530103,Futon_Bed\n\
530105,Futon_Sheet\n\
530107,Futon_Set\n\
530109,Futon_Seat_Set\n\
530111,Mattress\n\
530113,Pillow\n\
530197,General_Futon\n\
530201,Pure_Wool_Blanket\n\
530203,Synthetic_Fiber_Blanket\n\
530205,Cotton_Wool_Cloth\n\
530207,Bedclothes_Blanket\n\
530209,Electric_Blanket\n\
530211,Knee_Blanket\n\
530297,General_Blanket\n\
530301,Generality_Cover\n\
530303,Bed_Cover\n\
530305,Sheet_Cover\n\
530307,Blanket_Cover\n\
530309,Pillowcase\n\
530311,Collar_Cover\n\
530313,Sheet\n\
530397,Cover_Sheet\n\
530401,Futon_Bed\n\
530403,Bed_Pad\n\
530405,Bed_Spread\n\
530407,Bed_Blanket\n\
530409,Bed_Towelket\n\
530411,Bedsheet\n\
530413,Bed_Pillow\n\
530415,Bed\n\
530497,General_Bedding\n\
530501,Foot_Warmer_Futon\n\
530503,Foot_Warmer_Bed_Futon\n\
530505,Foot_Warmer_Futon_Cover\n\
530507,Foot_Warmer\n\
530509,Foot_Warmer_Blanket\n\
530511,Warm_Table\n\
530513,Chair\n\
530515,Futon_Seat\n\
530517,Futon_Seat_Cover\n\
530597,General_Foot_Warmer_Supplies\n\
530601,Baby_Futon_Set\n\
530603,Baby_Futon\n\
530605,Baby_Futon_Bed\n\
530607,Baby_Futon_Sheet\n\
530609,Baby_Blanket\n\
530611,Baby_Towelket\n\
530613,Cover_Sheet\n\
530615,Baby_Pillow\n\
530617,Baby_Bed\n\
530697,General_Baby_Bedding\n\
530701,Pajamas\n\
530703,Neglige\n\
530705,Gown\n\
530707,Ropes\n\
530709,Firewood\n\
530711,Poncho\n\
530713,Jinbei\n\
530797,General_Bedding\n\
530801,Bath_Towel\n\
530803,Face_Towel\n\
530805,Hand_Towel\n\
530807,Sport_Towel\n\
530809,Bathrobe\n\
530811,Restroom_Supplies\n\
530813,Towelket\n\
530897,General_Towel_Product\n\
530901,Curtains_Blind\n\
530903,Carpet_Rags\n\
530905,Mats\n\
530907,Cushion\n\
530909,Miscellanous_Interior_Decoration\n\
530911,Dining_Kitchen_Supplies\n\
530913,Bath_Supplies\n\
530915,Wall_Materials\n\
530997,General_Interior_Supplies\n\
539797,Other_Bedding_Bedclothings\n\
540101,Handkerchief\n\
540103,Mufflers_Scarf\n\
540105,Tie\n\
540107,Bowtie_Loop_Tie\n\
540109,Belt\n\
540111,Purse\n\
540113,Hat\n\
540197,General_Child_Clothing\n\
540201,Handkerchief\n\
540203,Mufflers_Scarf\n\
540205,Necktie\n\
540207,Bowtie_Ascot_Loop_Tie\n\
540209,Belt_Suspender\n\
540211,Glove\n\
540213,Hat\n\
540297,General_Gentlemen_Clothing\n\
540301,Handkerchiefs\n\
540303,Muffler\n\
540305,Necktie\n\
540307,Bowtie_Ascot_Loop_Tie\n\
540309,Belts_Suspender\n\
540311,Glove\n\
540313,Hat\n\
540397,General_Women_Clothing\n\
540401,Cloth_Belt\n\
540403,Lifting_Belt\n\
540405,Waist_String\n\
540407,Haori_String\n\
540409,Semi_Collar\n\
540411,Susoyoke\n\
540497,General_Kimono_Accessories\n\
540501,Handbag\n\
540503,Case\n\
540505,Fashion_Bag\n\
540597,General_Bag\n\
540601,Travel_Bag\n\
540603,Business_Bag\n\
540605,Student_Bag\n\
540607,Sports_Leisure_Bag\n\
540609,Musical_Instrument_Bag\n\
540611,Optical_Instrument_Radio_Bag\n\
540613,Professional_Use_Bag\n\
540697,General_Bag\n\
540701,Japanese_Umbrella\n\
540703,Western_Umbrella\n\
540705,Stick\n\
540797,Umbrella\n\
540801,Japanese_Fan\n\
540803,Utiwa\n\
540805,Japanese_Fan_Utiwa_Parts\n\
540897,General_Japanese_Fan_Utiwa\n\
540901,Ring\n\
540903,Necklace\n\
540905,Earring\n\
540907,Bracelets\n\
540909,Pendant\n\
540911,Brooches_Hatpins\n\
540913,Color_Buttons\n\
540915,Hair_Ornament\n\
540997,General_Ornament\n\
549701,Hygenic_Clothing\n\
549703,Leggins\n\
549705,Wallets\n\
549707,Smoking_Tools\n\
549797,Miscellaneous_Apparel\n\
550101,Shoes\n\
550104,Boots\n\
550107,Sandal\n\
550109,Other_Shoes\n\
550210,Child_Shoes\n\
550211,Junior_Shoes\n\
550212,Pumps\n\
550213,Shoes\n\
550214,Boots\n\
550216,Sport_Shoes\n\
550217,Sandal\n\
550218,High_Heels\n\
550219,Other_Shoes\n\
550320,Child_Shoes\n\
550321,Junior_Shoes\n\
550323,School_Shoes\n\
550324,Winter_Shoes\n\
550325,Lane_Shoes\n\
550326,Sport_Shoes\n\
550327,Sandals\n\
550329,Other_Shoes\n\
550430,Loafers_Leather_Sole\n\
550431,Loafers\n\
550432,Salon_Shoes_Leather_Sole\n\
550433,Salon_Shoes\n\
550434,Boots_Leather_Sole\n\
550435,Boots\n\
550436,Sport_Shoes\n\
550437,Sandals\n\
550439,Other_Shoes\n\
550541,Loafers\n\
550543,Salon_Shoes\n\
550544,Boots\n\
550546,Sport_Shoes\n\
550547,Sandal\n\
550548,Heppu\n\
550549,Other_Shoes\n\
550651,Standard_Casual_Shoes\n\
550654,Winter_Shoes\n\
550655,Lane_Shoes\n\
550656,Sport_Shoes\n\
550659,Other_Shoes\n\
550762,Pumps\n\
550763,Shoes\n\
550764,Boots\n\
550766,Sport_Shoes\n\
550767,Sandals\n\
550769,Other_Shoes\n\
550779,Miscellanous_Shoes\n\
550872,Pumps\n\
550873,Shoes\n\
550874,Boots\n\
550875,Lane_Shoes\n\
550876,Sport_Shoes\n\
550877,Sandal\n\
550878,Heppu\n\
550879,Other_Shoes\n\
550981,Standard_Casual_Shoes\n\
550984,Winter_Shoes\n\
550985,Lane_Shoes\n\
550986,Sport_Shoes\n\
550989,Other_Shoes\n\
551090,Accessories\n\
551091,Room_Shoes\n\
551092,Slipper\n\
551093,Straw_Sandals\n\
551094,Safety_Shoes\n\
551095,Work_Shoes\n\
551096,Sport_Shoes\n\
551097,Beach_Sandals\n\
560101,Baseball_Equipment\n\
560103,Softball_Equipment\n\
560105,Baseball_Shoes\n\
560107,Softball_Shoes\n\
560109,Baseball_Wear\n\
560111,Softball_Wear\n\
560197,General_Baseball_Softball_Equipment\n\
560201,Basketball_Equipment\n\
560203,Volleyball_Equipment\n\
560205,Rugby_Equipment\n\
560207,Soccer_Equipment\n\
560209,American_Football_Equipment\n\
560211,Handball_Equipment\n\
560213,Dodgeball_Equipment\n\
560215,Basketball_Shoes\n\
560217,Volleyball_Shoes\n\
560219,Rugby_Shoes\n\
560221,Soccer_Shoes\n\
560223,American_Football_Shoes\n\
560225,Handball_Shoes\n\
560227,Other_Ball_Game_Shoes\n\
560229,Basketball_Wear\n\
560231,Volleyball_Wear\n\
560233,Rugby_Wear\n\
560235,Soccer_Wear\n\
560237,American_Football_Wear\n\
560239,Handball_Wear\n\
560241,Other_Sportswear\n\
560297,General_Ball_Game_Equipment\n\
560301,Field_Equipment\n\
560303,Track_Equipment\n\
560305,Shoes\n\
560307,Wear\n\
560397,General_Field_Track_Equipment\n\
560401,Tennis_Equipment\n\
560403,Table_Tennis_Equipment\n\
560405,Badminton_Equipment\n\
560407,Tennis_Shoes\n\
560409,Table_Tennis_Shoes\n\
560411,Badminton_Shoes\n\
560413,Tennis_Wear\n\
560415,Table_Tennis_Wear\n\
560417,Badminton_Wear\n\
560497,General_Tennis_Equipment\n\
560501,Golf_Equipment\n\
560503,Field_Hokey_Equipment\n\
560505,Bowling_Equipment\n\
560507,Gateball_Equipment\n\
560509,Golf_Shoes\n\
560511,Field_Hockey_Shoes\n\
560513,Bowling_Shoes\n\
560515,Gateball_Shoes\n\
560517,Golf_Wear\n\
560519,Field_Hockey_Wear\n\
560521,Bowling_Wear\n\
560523,Gateball_Wear\n\
560597,General_Golf_Equipment\n\
560601,Ski_Equipment\n\
560603,Ice_Hockey_Equipment\n\
560605,Ice_Skating_Equipment\n\
560607,Roller_Skating_Equipment\n\
560609,Skiing_Shoes\n\
560611,Ice_Hockey_Shoes\n\
560613,Ice_Skating_Shoes\n\
560615,Roller_Skating_Shoes\n\
560617,Ski_Wear\n\
560619,Ice_Hockey_Wear\n\
560621,Ice_Skating_Wear\n\
560623,Roller_Skating_Wear\n\
560697,Skiing_Equipment\n\
560701,Boxing_Equipment\n\
560703,Bow_Equipment\n\
560705,Fencing_Equipment\n\
560707,Kenda_Equipment\n\
560709,Karate_Equipment\n\
560711,Halberd_Equipment\n\
560713,Train_Kendo_Equipment\n\
560715,Boxing_Shoes\n\
560717,Fencing_Shoes\n\
560719,Boxing_Wear\n\
560721,Fencing_Wear\n\
560723,Martial_Arts_Robe\n\
560797,Wrestling_Wear\n\
560801,Jumping_Box\n\
560803,Springs_Jump_Board\n\
560805,Parallel_Bars_Uneven_Bars\n\
560807,Balance_Beam\n\
560809,Iron_Bar\n\
560811,Flying_Rings\n\
560813,Pommel_Horse\n\
560815,Mat\n\
560817,Shoes\n\
560819,Wear\n\
560897,General_Gymnastics_Equipment\n\
560901,Fishing_Equipment\n\
560903,Swimming_Equipment\n\
560905,Ski_Snowboarding_Equipment\n\
560907,Surfing_Equipment\n\
560909,Board_Sailing_Equipment\n\
560911,Yacht_Motorboat_Equipment\n\
560913,Canoe_Equipment\n\
560915,Water_Skiing_Equipment\n\
560917,Fishing_Shoes\n\
560919,Yachts_Motorboat_Shoes\n\
560921,Fishing_Rod\n\
560923,Swimwear\n\
560925,Diving_Motorboat_Wear\n\
560927,Surfing_Wear\n\
560929,Board_Sailing_Wear\n\
560931,Yacht_Wear\n\
560933,Canoe_Wear\n\
560935,Water_Ski_Wear\n\
560997,General_Fishing_Equipment\n\
569701,Hiking_Equipment\n\
569703,Cycling_Equipment\n\
569705,Jogging_Walking_Equipment\n\
569707,Training_Equipment\n\
569709,Shooting_Equipment\n\
569711,Horseback_Riding_Equipment\n\
569713,Hiking_Shoes\n\
569715,Cycling_Shoes\n\
569717,Jogging_Walking_Shoes\n\
569719,Training_Shoes\n\
569721,Shooting_Shoes\n\
569723,Horseback_Riding_Shoes\n\
569725,Other_Shoes\n\
569727,Hiking_Wear\n\
569729,Cycling_Wear\n\
569731,Jogging_Walking_Wear\n\
569733,Training_Wear\n\
569735,Shooting_Wear\n\
569737,Horseback_Riding_Wear\n\
569739,Other_Wear\n\
569797,Miscellaneous_Sporting_Goods\n";
